;**************************************************************************
;
;                 PVK40 + K2 + MU Beta example : Display
;
; Processor : PIC16F84A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F84A   ; processor type
        __config 0x3FF1         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
indf    equ     0x00            ;indirect addressing
pcl     equ     0x02
status  equ     0x03            ;status is on the 0x03 address
fsr     equ     0x04            ;file select register
porta   equ     0x05
trisa   equ     0x05            ;direct addressing
portb   equ     0x06
trisb   equ     0x06            ;direct addressing
; General Purpose Register definition :
tmp     equ     0x10
cnt1    equ     0x11            ;used for Wait subroutine
; Destination definition :
w       equ     0x00
f       equ     0x01
; Bits definition :
#define RP0     status,5        ;RP0 is bit 5 in status register
#define Z       status,2        ;Z is bit 2 in status register
#define DCOM    porta,1         ;display common anode
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  movlw   0xFF            ;set portb
        movwf   portb
        bsf     RP0             ;bank 1
        movlw   B'11111101'     ;pin RA1 is output
        movwf   trisa
        bcf     RP0             ;bank 0
        movlw   0x86            ;0x86 = trisb address
        movwf   fsr
;
Main:   movlw   0xFF            ;portb pins are inputs
        movwf   indf            ;write to trisb
        bsf     DCOM            ;turn off the display
        call    Wait            ;delay to stable inputs
        movf    portb,w         ;read portb (pushbuttons)
        bcf     DCOM            ;turn on the display
        clrf    indf            ;portb pins are outputs
        movwf   tmp
;
        movlw   0x08
Test:   btfss   tmp,7           ;detect which button is pressed
        goto    Disp
        rlf     tmp,f
        addlw   0xFF            ;subtract 1
        btfss   Z
        goto    Test
        movlw   0x0A            ;no button pressed, clear display
;
Disp:   call    Table           ;convert 0-0x0A to char 0-9
        movwf   portb           ;write to port
        call    Wait
        goto    Main
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 + K2 + MU Beta example : Display
        end
