;**************************************************************************
;
;                 PVK40 + K2 + MU Beta example : Counter
;
; Processor : PIC16F84A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F84A   ; processor type
        __config 0x3FF1         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
tmr0    equ     0x01            ;timer 0
optreg  equ     0x01            ;direct addressing
pcl     equ     0x02
status  equ     0x03            ;status is on the 0x03 address
porta   equ     0x05
trisa   equ     0x05            ;direct addressing
portb   equ     0x06
trisb   equ     0x06            ;direct addressing
intcon  equ     0x0B
; General Purpose Register definition :
bckwreg equ     0x10            ;working register backup during interrupt
intcnt  equ     0x11            ;interrupt counting
disp1   equ     0x12            ;digit '1'
disp2   equ     0x13            ;digit '10'
disp3   equ     0x14            ;digit '100'
disp4   equ     0x15            ;digit '1000'
cnt1    equ     0x16            ;used for Wait subroutine
; Destination definition :
w       equ     0x00
f       equ     0x01
; Bits definition :
#define RP0     status,5        ;RP0 is bit 5 in status register
#define Z       status,2        ;Z is bit 2 in status register
#define C       status,0        ;C is bit 0 in status register
#define T0IF    intcon,2        ;TMR0 interrupt flag
#define GIE     intcon,7        ;Global Interrupt Enable bit
#define DCOM1   porta,1         ;display common anode 1
#define DCOM2   porta,2         ;display common anode 2
#define DCOM3   porta,3         ;display common anode 3
#define DCOM4   porta,4         ;display common anode 4
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        bcf     T0IF            ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_D           ;then return
;
        movlw   0x19            ;else reinitialise intcnt register
        movwf   intcnt
        incf    disp1,f         ;increment disp1
        movlw   0x0A
        xorwf   disp1,w         ;if disp1 > 9 
        btfss   Z
        goto    Int_A
        clrf    disp1           ;then disp1 = 0
        incf    disp2,f         ;and increment disp2
Int_A:  movlw   0x0A
        xorwf   disp2,w         ;if disp2 > 9 
        btfss   Z
        goto    Int_B
        clrf    disp2           ;then disp2 = 0
        incf    disp3,f         ;and increment disp3
Int_B:  movlw   0x0A
        xorwf   disp3,w         ;if disp3 > 9 
        btfss   Z
        goto    Int_C
        clrf    disp3           ;then disp3 = 0
        incf    disp4,f         ;and increment disp4
Int_C:  movlw   0x0A
        xorwf   disp4,w         ;if disp4 > 9 then disp4 = 0
        btfsc   Z
        clrf    disp4
Int_D:  movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0xFF
        movwf   portb           ;set portb
        movwf   porta           ;set porta
        bsf     RP0             ;bank 1
        movlw   B'11100001'     ;pins RA1,2,3,4 are outputs
        movwf   trisa
        clrf    trisb           ;portb pins are outputs
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   optreg
        bcf     RP0             ;bank 0
        clrf    disp1
        clrf    disp2
        clrf    disp3
        clrf    disp4
        movlw   0x19            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     GIE             ;enable global interrupt
;
Main:   movf    disp1,w
        call    Table
        movwf   portb
        bcf     DCOM1           ;turn on display 1
        call    Wait
        bsf     DCOM1           ;turn off display 1
        movf    disp2,w
        call    Table
        movwf   portb
        bcf     DCOM2           ;turn on display 2
        call    Wait
        bsf     DCOM2           ;turn off display 2
        movf    disp3,w
        call    Table
        movwf   portb
        bcf     DCOM3           ;turn on display 3
        call    Wait
        bsf     DCOM3           ;turn off display 3
        movf    disp4,w
        call    Table
        movwf   portb
        bcf     DCOM4           ;turn on display 4
        call    Wait
        bsf     DCOM4           ;turn off display 4
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 + K2 + MU Beta example : Counter
        end
