;**************************************************************************
;
;                 PVK40 + K2 + MU Beta example : A/D Converter 2
;
; Processor : PIC16C710
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on TRIM1 on S9
; 3) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16C710   ; processor type
        __config 0x3FF9         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16c710.INC>
bckwreg equ     0x10            ;working register backup during interrupt
intcnt  equ     0x11            ;interrupt counting
disp1   equ     0x12            ;digit '1'
disp2   equ     0x13            ;digit '10'
disp3   equ     0x14            ;digit '100'
disp4   equ     0x15            ;digit '1000'
param0  equ     0x16            ;used in Wto5d subroutine
param1  equ     0x17            ;used in Wto5d subroutine
param2  equ     0x18            ;used in Wto5d subroutine
param3  equ     0x19            ;used in Wto5d subroutine
param4  equ     0x1A            ;used in Wto5d subroutine
count   equ     0x1B            ;used in Wto5d subroutine
temp    equ     0x1C            ;used in Wto5d subroutine
cnt1    equ     0x1D            ;used for Wait subroutine
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_A           ;then return
;
        movlw   0x0B            ;else reinitialise intcnt register
        movwf   intcnt
        bsf     adcon0,GO       ;start A/D conversion
Adw:    btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw
        movf    adres,W         ;copy adres to param0
        movwf   param0
        clrf    param1
        call    Wto5d           ;convert to decimal number
        movf    param2,W        ;save result to disp1-disp4
        andlw   0x0F
        movwf   disp1
        swapf   param2,W
        andlw   0x0F
        movwf   disp2
        movf    param3,W
        andlw   0x0F
        movwf   disp3
        swapf   param3,W
        andlw   0x0F
        movwf   disp4
Int_A:  movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0xFF
        movwf   porta           ;set porta
        movwf   portb           ;set portb
        bsf     status,RP0      ;bank 1
        movlw   B'00000010'     ;pin RA0 is analog input
        movwf   adcon1
        movlw   B'11100001'     ;pins RA1,2,3,4 are outputs
        movwf   trisa
        clrf    trisb           ;portb pins are outputs
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
        movlw   0x0A
        movwf   disp1
        movwf   disp2
        movwf   disp3
        movwf   disp4
        movlw   B'01000001'     ;enable A/D converter module
        movwf   adcon0
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
;
Main:   movf    disp1,w
        call    Table
        movwf   portb
        movlw   B'11111100'     ;turn on display 1
        movwf   porta
        call    Wait
        movlw   B'11111110'     ;turn off display 1
        movwf   porta
        movf    disp2,w
        call    Table
        movwf   portb
        movlw   B'11111010'     ;turn on display 2
        movwf   porta
        call    Wait
        movlw   B'11111110'     ;turn off display 2
        movwf   porta
        movf    disp3,w
        call    Table
        movwf   portb
        movlw   B'11110110'     ;turn on display 3
        movwf   porta
        call    Wait
        movlw   B'11111110'     ;turn off display 3
        movwf   porta
        movf    disp4,w
        call    Table
        movwf   portb
        movlw   B'11101110'     ;turn on display 4
        movwf   porta
        call    Wait
        movlw   B'11111110'     ;turn off display 4
        movwf   porta
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Wto5d:  bcf     status,C        ;this subroutine converts 16-bit number
        clrf    count           ;in param0(LSB) and param1(MSB) to
        bsf     count,4         ;5-digit decimal number in param2-4
        clrf    param4
        clrf    param3
        clrf    param2
Wto5d2: rlf     param0,F
        rlf     param1,F
        rlf     param2,F
        rlf     param3,F
        rlf     param4,F
        decf    count,F
        btfsc   status,Z
        return
        movf    param2,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param2
        movf    param2,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
	movwf   param2
        movf    param3,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param3
        movf    param3,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
        movwf   param3
        movf    param4,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param4
        movf    param4,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
        movwf   param4
        goto    Wto5d2
;**************************************************************************
; end of PVK40 + K2 + MU Beta example : A/D Converter 2
        end
