;**************************************************************************
;
;         PVK40 + K2 + MU Beta example : Serial EEPROM 93xx (SW)
;
; Processor : PIC16F627
; Clock : INTIO 4 MHz
;
; On your PVK40 board :
; 1) Insert 93LC46B into 93xx socket
; 2) Switch off JP1 jumper
; 3) Switch off all DIP switches except:
; 4) Switch on B0,1,2,4 DISP on S11
; 5) Switch on B3 LED on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F627   ; processor type
        __config 0x3F70         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f627.INC>
; General Purpose Register definition :
intcnt  equ     0x20            ;interrupt counting
cnt1    equ     0x21            ;used in Wait subroutine
dig1    equ     0x22            ;digit '1'
dig2    equ     0x23            ;digit '10'
dig3    equ     0x24            ;digit '100'
dig4    equ     0x25            ;digit '1000'
eedat0  equ     0x26            ;EEPROM data 0 (LSB)
eedat1  equ     0x27            ;EEPROM data 1 (MSB)
eeadrs  equ     0x28            ;used in Wree and Rdee subroutines
digtmp1 equ     0x29            ;store dig1 value
digtmp2 equ     0x2A            ;store dig2 value
swrtmp0 equ     0x2B            ;used in Swree subroutine
swrtmp1 equ     0x2C            ;used in Swree subroutine
buffer  equ     0x2D            ;used in Output subroutine
bufcnt  equ     0x2E            ;used in Output subroutine
ledfl   equ     0x2F            ;bit0=LED flag
bckwreg equ     0x70            ;working register backup during interrupt
bcksta  equ     0x71            ;status register backup during interrupt
; Bits definition :
#define EECS    porta,0         ;EEPROM chip select
#define EEDI    porta,6         ;EEPROM data in
#define EEDO    portb,2         ;EEPROM data out
#define EECLK   porta,7         ;EEPROM clock
#define DCOM1   porta,1         ;display common anode 1
#define DCOM2   porta,2         ;display common anode 2
#define DCOM3   porta,3         ;display common anode 3
#define DCOM4   porta,4         ;display common anode 4
#define LED     portb,4         ;LED
#define PB0     portb,0         ;pushbutton 0
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        bcf     status,RP0      ;bank 0
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_D           ;then return
;
        movlw   0x0B            ;else reinitialise intcnt register
        movwf   intcnt
        incf    dig1,f          ;increment dig1
        movlw   0x0A
        subwf   dig1,w          ;if dig1 > 9 
        btfss   status,C
        goto    Int_A
        clrf    dig1            ;then dig1 = 0
        incf    dig2,f          ;and increment dig2
Int_A:  movlw   0x0A
        subwf   dig2,w          ;if dig2 > 9 
        btfss   status,C
        goto    Int_B
        clrf    dig2            ;then dig2 = 0
        incf    dig3,f          ;and increment dig3
Int_B:  movlw   0x0A
        subwf   dig3,w          ;if dig3 > 9 
        btfss   status,C
        goto    Int_C
        clrf    dig3            ;then dig3 = 0
        incf    dig4,f          ;and increment dig4
Int_C:  movlw   0x0A
        subwf   dig4,w          ;if dig4 > 9 then dig4 = 0
        btfsc   status,C
        clrf    dig4
Int_D:  swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  clrf    portb           ;clear portb latch
        movlw   B'00011110'     ;setup porta pin latches
        movwf   porta
        movlw   B'00000111'     ;porta pins are digital
        movwf   cmcon
        bsf     status,RP0      ;bank 1
        movlw   B'01100000'     ;pin RA5,6 aer inputs
        movwf   trisa
        clrf    trisb           ;portb pins are outputs
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
;
        movlw   0x00            ;read address 0
        call    Rdee            ;from EEPROM
        movf    eedat1,w
        andlw   0x0F
        movwf   dig4            ;save to dig4
        movf    eedat0,w
        andlw   0x0F
        movwf   dig3            ;save to dig3
        movlw   0x01            ;read address 1
        call    Rdee            ;from EEPROM
        movf    eedat1,w
        andlw   0x0F
        movwf   dig2            ;save to dig2
        movf    eedat0,w
        andlw   0x0F
        movwf   dig1            ;save to dig1
        call    Ewen            ;EEPROM write enable
;
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
        movlw   0x86            ;0x86 = trisb address
        movwf   fsr
;
Main:   movf    dig1,w
        call    Table
        movwf   portb
        bcf     DCOM1           ;turn on display 1
        call    Wait
        bsf     DCOM1           ;turn off display 1
        movf    dig2,w
        call    Table
        movwf   portb
        bcf     DCOM2           ;turn on display 2
        call    Wait
        bsf     DCOM2           ;turn off display 2
        movf    dig3,w
        call    Table
        movwf   portb
        bcf     DCOM3           ;turn on display 3
        call    Wait
        bsf     DCOM3           ;turn off display 3
        movf    dig4,w
        call    Table
        movwf   portb
        bcf     DCOM4           ;turn on display 4
        call    Wait
        bsf     DCOM4           ;turn off display 4
;
        bcf     LED             ;LED = ledfl,0
        btfsc   ledfl,0
        bsf     LED
        bsf     indf,0          ;RB0 is input
        call    Wait            ;delay to stable inputs
        btfss   PB0             ;if pushbutton is pressed
        bsf     ledfl,0         ;turn on the LED (flag)
        btfsc   PB0
        bcf     ledfl,0
        bcf     indf,0          ;RB0 is output
;
        btfss   ledfl,0         ;if the LED is on
        goto    Main
        bcf     intcon,GIE      ;disable interrupt
        movf    dig1,w          ;save dig1-dig4 state
        movwf   digtmp1
        movf    dig2,w
        movwf   digtmp2
        movf    dig3,w
        movwf   eedat0
        movf    dig4,w
        movwf   eedat1
        bsf     intcon,GIE      ;enable interrupt
        movlw   0x00
        call    Swree           ;write to EEPROM
        movf    digtmp1,w
        movwf   eedat0
        movf    digtmp2,w
        movwf   eedat1
        movlw   0x01
        call    Swree           ;write to EEPROM
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Output: movwf   buffer          ;place data in buffer
        movlw   0x08
        movwf   bufcnt
Outpt2: btfss   buffer,7        ;EEDO := buffer MSb
        bcf     EEDO
        btfsc   buffer,7
        bsf     EEDO
        bsf     EECLK           ;EECLK = 1
        nop
        bcf     EECLK           ;EECLK = 0
        btfss   EEDI            ;buffer MSb := EEDI
        bcf     buffer,7
        btfsc   EEDI
        bsf     buffer,7
        rlf     buffer,f
        decfsz  bufcnt,f
        goto    Outpt2
        rlf     buffer,f
        return
;
Ewen:   bsf     EECS            ;set chip select line high
        movlw   B'00000001'     ;start bit
        call    Output
        movlw   B'00110000'     ;opcode for EWEN
        call    Output
        bcf     EECS            ;clear chip select line
        return
;
Wree:   bsf     EECS            ;set chip select line high
        movwf   eeadrs
        movlw   B'00000001'     ;start bit
        call    Output
        movf    eeadrs,w        ;opcode + addr. for WRITE
        andlw   B'00111111'
        iorlw   B'01000000'
        call    Output
        movf    eedat1,w        ;data 15-8
        call    Output
        movf    eedat0,w        ;data 7-0
        call    Output
        bcf     EECS            ;clear chip select line
        nop                     ;EECS must be low for > 250 ns
        bsf     EECS            ;set chip select line high
Wreew:  btfss   EEDI            ;test ready/busy status
        goto    Wreew
        bcf     EECS            ;clear chip select line
        return
;
Rdee:   bsf     EECS            ;set chip select line high
        movwf   eeadrs
        movlw   B'00000001'     ;start bit
        call    Output
        movf    eeadrs,w        ;opcode + addr. for READ
        andlw   B'00111111'
        iorlw   B'10000000'
        call    Output
        movlw   0x00
        call    Output          ;8x clk
        movf    buffer,w        ;read received data
        movwf   eedat1          ;save as bits 15-8
        movlw   0x00
        call    Output          ;8x clk
        movf    buffer,w        ;read received data
        movwf   eedat0          ;save as bits 7-0
        bcf     EECS            ;clear chip select line
        return
;
Swree:  movwf   eeadrs          ;this subroutine writes to EEPROM
        movf    eedat0,w        ;use this subroutine to avoid multiply
        movwf   swrtmp0         ;rewriting one address with the same value
        movf    eedat1,w
        movwf   swrtmp1
        movf    eeadrs,w
        call    Rdee            ;read EEPROM data
        movf    eedat0,w        ;compare data in EEPROM
        xorwf   swrtmp0,w       ;with data to write
        btfss   status,Z
        goto    Swreew
        movf    eedat1,w
        xorwf   swrtmp1,w
        btfsc   status,Z
        return
Swreew: movf    swrtmp0,w
        movwf   eedat0
        movf    swrtmp1,w
        movwf   eedat1
        movf    eeadrs,w
        call    Wree            ;write to EEPROM
        return
;**************************************************************************
; end of PVK40 + K2 + MU Beta example : Serial EEPROM 93xx (SW)
        end
