;**************************************************************************
;
;                 PVK40 + K2 + MU Beta example : Sound-level-meter
;
; Processor : PIC16C710
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on C1 BARGRAPH on S11
; 3) Switch on A5 MIC on S9
;**************************************************************************
; Assembler directives :
        list    p = PIC16C710   ; processor type
        __config 0x3FF9         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16c710.INC>
smpcnt  equ     0x10            ;used in Sndm subroutine
peak    equ     0x11            ;maximum
down    equ     0x12            ;minimum
bargr   equ     0x13            ;bargraph data
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  clrf    porta           ;clear porta latch
        movlw   0xFF
        movwf   portb           ;set portb latch
        bsf     status,RP0      ;bank 1
        movlw   B'00000010'     ;pin RA0 is analog input
        movwf   adcon1
        movlw   B'11101111'     ;pin RA4 is output
        movwf   trisa
        clrf    trisb           ;portb pins are outputs
        movlw   B'11010000'     ;prescaler assigned to TMR0
        movwf   option_reg      ;TMR0 period = 0.625 ms
        bcf     status,RP0      ;bank 0
        movlw   B'01000001'     ;enable A/D converter module
        movwf   adcon0
        clrf    tmr0
        bcf     intcon,T0IF     ;clear timer0 overflow flag
;
Main:   call    Sndm            ;measure maximum and minimum value
        movf    down,W          ;peak - down to peak
        subwf   peak,F
        movlw   0xFF
        movwf   bargr
;
        movlw   0x16            ;constant = 22
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,0         ;switch on bargraph segment 1
;
        movlw   0x20            ;constant = 32
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,1         ;switch on bargraph segment 2
;
        movlw   0x2D            ;constant = 45
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,2         ;switch on bargraph segment 3
;
        movlw   0x40            ;constant = 64
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,3         ;switch on bargraph segment 4
;
        movlw   0x4E            ;constant = 78
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,4         ;switch on bargraph segment 5
;
        movlw   0x80            ;constant = 128
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,5         ;switch on bargraph segment 6
;
        movlw   0xA0            ;constant = 160
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,6         ;switch on bargraph segment 7
;
        movlw   0xBE            ;constant = 190
        subwf   peak,W
        btfsc   status,C        ;if peak > constant
        bcf     bargr,7         ;switch on bargraph segment 8
;
        movf    bargr,W         ;write to bargraph
        movwf   portb
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Sndm:   clrf    peak            ;detect maximum and minimum value
        movlw   0xFF
        movwf   down
        movlw   0xFF            ;255 samples
        movwf   smpcnt
Sndm_A: bsf     adcon0,GO       ;starts A/D conversion
Sndm_B: btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Sndm_B
        btfss   intcon,T0IF     ;wait for TMR0 overflow
        goto    Sndm_B
        bcf     intcon,T0IF     ;clear timer0 overflow flag
;
        movf    adres,w
        subwf   peak,w
        btfsc   status,C
        goto    Sndm_C
        movf    adres,W         ;if adres > peak
        movwf   peak            ;save adres to peak
;
Sndm_C: movf    adres,W
        subwf   down,W
        btfss   status,C
        goto    Sndm_D
        movf    adres,W         ;if adres < down
        movwf   down            ;save adres to down
Sndm_D: decfsz  smpcnt,F        ;loop
        goto    Sndm_A
        return
;**************************************************************************
; end of PVK40 + K2 + MU Beta example : Sound-level-meter
        end
