/*============================================================================

                 PVK40 + K2 + MU Beta example : Counter

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F84A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on B0,1,2,4 DISP on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & PWRTEN & WDTDIS & XT);


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

typedef unsigned int clock_t;

char digit[5] = {0xFF,0xFD,0xFB,0xF7,0xEF}; 
char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;         /* systimer is incremented every 20ms */
clock_t now;
clock_t lap;
unsigned int period;
unsigned int count=0;

/*============================================================================

       Program

============================================================================*/

void interrupt timer0(void)
{
  T0IF=0;
  systimer++;
}

clock_t clock(void)        /* read systimer */
{
  clock_t temp;

  GIE=0;
  temp=systimer;
  GIE=1;
  return temp;
}

/* dispdigit(j,p) - shows the value of an integer j
   on one p of the 7 segments display            */ 
void dispdigit(int j, int p)
{
  PORTA=digit[0];
  PORTB=segment[j];
  TRISB=0x00;
  PORTA=digit[p];
}


void main(void)
{
  TRISA=0xE1;
  /* initialisation of timer0 */
	
  PS2=1;  PS1=0;  PS0 = 1; /* select prescaler by 64 */
  T0CS = 0;                /* select timer0 */
  PSA=0;                   /* select prescaler for timer0 */
  TMR0 =0;                 /* preload timer0 */
  T0IE = 1;                /* interrupts are actived */
  GIE=1;                   /* interrupts are actived */
  period=50;               /* 1 second */
  /*--------------------------*/

  for(;;) {
    now=clock();
    if ((now-lap)>period) {
      lap+=period;
      if(++count>9999)     /*counting*/
        count=0;
    }
    dispdigit(count%10,1);
    dispdigit((count/10)%10,2);
    dispdigit((count/100)%10,3);
    dispdigit(count/1000,4);
  } // end of the for loop
}