/*============================================================================

           PVK40 + K2 + MU Beta example : Serial EEPROM 24xx (SW)

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F627
 Clock : INTIO 4MHz

 On your PVK40 board set DIP switches according to following way:
 1) Insert 24LC01B into 24xx socket
 2) Switch off all DIP switches except:
 3) Switch on B0,1,2,4 DISP on S11
 4) Switch on B3 LED on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & LVPDIS & BOREN & PWRTEN & WDTDIS & INTIO); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

typedef unsigned int clock_t;

static bit led @ PORTBIT(PORTB,4);
static bit pushbutton0 @ PORTBIT(PORTB,0);
static bit i2c_sda @ PORTBIT(PORTA,6);
static bit i2c_scl @ PORTBIT(PORTA,7);
static bit i2c_sda_tris @ PORTBIT(TRISA,6);
static bit i2c_scl_tris @ PORTBIT(TRISA,7);

char digit[5] = {0x3F,0xFD,0xFB,0xF7,0xEF};
char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;         /* systimer is incremented every 16.4ms */
clock_t now;
clock_t lap;
unsigned int period;
unsigned int count;

/*============================================================================

       Program

============================================================================*/

void interrupt timer0(void)
{
  T0IF=0;
  systimer++;
}

clock_t clock(void)        /* read systimer */
{
  clock_t temp;

  GIE=0;
  temp=systimer;
  GIE=1;
  return temp;
}

void dispdigit(int j, int p)
{
  PORTA=digit[0];
  PORTB=segment[j];
  TRISB=0x00;
  PORTA=digit[p];
}

void timer0init(void)
{    /* initialisation of timer0 */
  PS2=1;  PS1=0;  PS0 = 1; /* select prescaler by 64 */
  T0CS = 0;                /* select timer0 */
  PSA=0;                   /* select prescaler for timer0 */
  TMR0 =0;                 /* preload timer0 */
  T0IE = 1;                /* interrupts are actived */
}

void portinit(void)
{
  PORTA=0x3F;
  PORTB=0x00;
  CMCON=0x07;
  TRISA=0xE1;
  TRISB=0x00;
}

void delayus(unsigned char us)
{
  for (;us--;);
}

void i2c_start_bit(void)
{
  i2c_sda_tris=1;  /* set data bus */
  delayus(1);
  i2c_scl_tris=1;  /* set clock bus */
  delayus(2);
  i2c_sda_tris=0;  /* reset data bus */
  delayus(2);
  i2c_scl_tris=0;  /* reset clock bus */
}

void i2c_stop_bit(void)
{
  i2c_scl_tris=0;  /* reset clock bus */
  delayus(1);
  i2c_sda_tris=0;  /* reset data bus */
  delayus(2);
  i2c_scl_tris=1;  /* set clock bus */
  delayus(2);
  i2c_sda_tris=1;  /* set data bus */
}

unsigned char i2c_write(unsigned char data)
/* send byte to i2c device 
   returned value is acknowledge flag */
{ 
  unsigned char mask, flag;

  for(mask=0x80;mask!=0;mask>>=1) {
    if(data&mask)
      i2c_sda_tris=1;
    else
      i2c_sda_tris=0;
    delayus(2);
    i2c_scl_tris=1;  /* set clock bus */
    delayus(2);
    i2c_scl_tris=0;  /* reset clock bus */
    delayus(2);
  }
  i2c_sda_tris=1;  /* set data bus */
  delayus(2);
  i2c_scl_tris=1;  /* set clock bus - 9. clock pulse */
  delayus(2);
  flag=i2c_sda; /* read ACKF */
  i2c_scl_tris=0;  /* reset clock bus */
  delayus(2);
  return flag;
}

unsigned char i2c_read(unsigned char ackdata)
{  /* read byte from i2c bus */
  unsigned char temp, bitcnt;

  i2c_sda_tris=1;  /* set data bus (Hi-Impedance) */
  temp=0x00;
  delayus(1);
  for(bitcnt=8;bitcnt!=0;bitcnt--) {
    i2c_scl_tris=1;  /* set clock bus */
    delayus(1);
    temp<<=1;
    temp|=i2c_sda; /* read data */
    delayus(1);
    i2c_scl_tris=0;  /* reset clock bus */
    delayus(1);
  }
  if(ackdata) 
    i2c_sda_tris=1;
  else
    i2c_sda_tris=0;
  delayus(1);
  i2c_scl_tris=1;  /* set clock bus - send ACK */
  delayus(2);
  i2c_scl_tris=0;  /* reset clock bus */
  delayus(1);
  return temp;
}

void eep_wrt(unsigned char address, unsigned char data)
{
  for(;;) {     /* acknowledge pooling loop */
    i2c_start_bit();  /* start bit */
    if(!i2c_write(0xA0))  /* control byte - write */
      break;
    i2c_stop_bit();   /* stop bit */
  }
  i2c_write(address); /* write address to EEPROM */
  i2c_write(data);  /* write data to EEPROM */
  i2c_stop_bit(); /* stop bit */
}

unsigned char eep_read(unsigned char address)
{
  unsigned char tempdata;

  for(;;) {     /* acknowledge pooling loop */
    i2c_start_bit();  /* start bit */
    if(!i2c_write(0xA0))  /* control byte - write */
      break;
    i2c_stop_bit();   /* stop bit */
  }
  i2c_write(address); /* write address to EEPROM */
  i2c_start_bit();
  i2c_write(0xA1);  /* control byte - read */
  tempdata=i2c_read(1);  /* select NO_ACK bit */
  i2c_stop_bit(); /* stop bit */
  return tempdata;
}

void eep_write(unsigned char address, unsigned char data)
{
  if (eep_read(address)!=data)
    eep_wrt(address,data);
}

void main(void)
{
  portinit();
  timer0init();
  period=61;               /* app. 1 second */
  GIE=1;                   /* interrupts are actived */

  count=eep_read(0x00)+(eep_read(0x01)<<8);
  if(count>9999)
    count=0;

  for(;;) {
    now=clock();
    if ((now-lap)>period) {
      lap+=period;
      if(++count>9999)     /*counting*/
        count=0;
    }
    dispdigit(count%10,1);
    delayus(250);
    dispdigit((count/10)%10,2);
    delayus(250);
    dispdigit((count/100)%10,3);
    delayus(250);
    dispdigit(count/1000,4);
    delayus(250);

    TRISB=0xFF;   /* inputs */
    PORTA=digit[0];
    delayus(5);
    if(!pushbutton0) {
      TRISB=0x00;
      led=1;
      delayus(100);
      eep_write(0x00,count&0xFF);
      eep_write(0x01,count>>8);
    }
    else {
      led=0;
      delayus(250);
    }
  } // end of the for loop
}


