/*============================================================================

               PVK40 + K2 + MU Beta example : PWM

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F627
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on C2 BUZZER on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>

/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & PWRTEN & LVPDIS & WDTDIS & XT);


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

static bit pushbutton0 @ PORTBIT(PORTB,0);
static bit pushbutton1 @ PORTBIT(PORTB,1);
static bit pushbutton2 @ PORTBIT(PORTB,2);


/*============================================================================

       Program

============================================================================*/

void pwminit(void)
{
  CCPR1H=0x00;
  CCPR1L=0x00;
  CCP1CON=0x0F;
  TMR2=0x00;
  TMR2ON=1;
  PR2=0xFF;
  TRISB&=0xF7;
}

void tone(unsigned int volume)
{
  if (volume&1)
    CCP1Y=1;
  else
    CCP1Y=0;
  if (volume&2)
    CCP1X=1;
  else
    CCP1X=0;
  CCPR1L=(volume>>2)&0xFF;
}

int main(void)
{
  int tmp;

  pwminit();
  for(;;) {
    tmp=0x0000;
    if(!pushbutton0)
      tmp=16;  /* 1.6% */
    if(!pushbutton1)
      tmp=128;  /* 12% */
    if(!pushbutton2)
      tmp=512;  /* 50% */
    tone(tmp);
  }
}
