/*============================================================================

     lcd.c - handling routines for LCD module MC1602

 Board : PVK40
 Processor : PIC16F627
 Clock : INTIO 4MHz

============================================================================*/

#include <pic.h>
#include "lcd.h"

#define cbi(X,Y)		((X)&=~(1<<(Y)))
#define sbi(X,Y)		((X)|=1<<(Y))

unsigned char PORTB_LATCH;
unsigned char PORTA_LATCH;

//approximate delay loop (1 instruction cycle lasts cca 1 us)
void MicroDelay(unsigned short us)
{
  while (us--);
}

void DispEn(void)
{
  PORTA=PORTA_LATCH|=0x08;
  PORTA=PORTA_LATCH&=0xf7;
}

void DispWr(unsigned char c, unsigned char isdata)
{
  PORTA_LATCH=PORTA;
  if (isdata)
    PORTA=PORTA_LATCH|=0x40;
  else
    PORTA=PORTA_LATCH&=0x3F;

  PORTB=c;
  DispEn();
}

void DispWrByte(unsigned char c, unsigned char isdata, unsigned short delay_us)
{
  DispWr(c,isdata);
  MicroDelay(delay_us);
}

/*=====================================================*/
/* Display hardware initialisation */
void DispHwInit(void)
{
  /* RS */
  cbi(PORTA,6);
  cbi(TRISA,6);

  /* read/write */
  cbi(PORTA,7);
  cbi(TRISA,7);

  /* enable */
  cbi(PORTA,3);
  cbi(TRISA,3);

  /* RA digital */
  CMCON=0x07;

  /* data */
  TRISB=0;

  /*
  force the display to go into 8-bit mode
  since it might be in 4-bit mode and unsynchronized, thus we should better
  wait wait for possible instruction to finish after first DispWr()
  */
  DispWr(0x30,0);
  MicroDelay(2000);
  DispWr(0x30,0);
  MicroDelay(50);
  DispWr(0x30,0);
  MicroDelay(50);

  DispWrByte(0x38,0,50); //8-bit mode, 2 lines, 5x8 matrix

  DisplayClear();
  DisplayOnOff(DISPLAY_ONOFF|CURSOR_LINE|CURSOR_BLINK); //restore display state

  DispWrByte(0x06,0,50); //entry mode - move to right, no display shift
}

