/*============================================================================

 PVK40 + K2 + MU Beta example : Infra-red Reflective photoelectric sensor

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F627
 Clock : INTIO 4 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on B3 LED on S11
 3) Switch on B5 IRF_TX on S11
 4) Switch on C0 IRF_RX on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & LVPDIS & BOREN & PWRTEN & WDTDIS & INTIO); 


/*============================================================================

       User definition

============================================================================*/

#define TMR0PRE 0xED  /* timer0 preload value */

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

static bit ir_rx @ PORTBIT(PORTB,1);
static bit ir_tx @ PORTBIT(PORTA,6);
static bit led @ PORTBIT(PORTB,4);


/*============================================================================

       Program

============================================================================*/

void timer0init(void)
{ /* initialisation of timer0; */
	T0CS=0; /* select timer0 clock */
	PSA=1;  /* select prescaler for wdt */
	TMR0|=TMR0PRE; /* preload timer0 */
}

int main(void)
{
  TRISA=0xBF;
  TRISB=0xEF;
  timer0init();

  for(;;) {
    led=ir_rx;
    ir_tx=1;
    asm("nop");
    ir_tx=0;
    while(!T0IF) ; /* wait for timer0 overflow */
    TMR0|=TMR0PRE; /* preload timer0 */
    T0IF=0;  /* clear flag */
  }
}
